@@ -1,4 +1,5 @@
-# Copyright 2013-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2018-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,15 +14,12 @@
 # limitations under the License.
 #
 # Authors:
-# - Diego Ciangottini <diego.ciangottini@gmail.com>, 2018
+# - dciangot <diego.ciangottini@cern.ch>, 2018
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-
-try:
-    from urllib import quote_plus
-except ImportError:
-    from urllib.parse import quote_plus
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2021
 
 from requests.status_codes import codes
+from six.moves.urllib.parse import quote_plus
 
 from rucio.client.baseclient import BaseClient
 from rucio.client.baseclient import choice
@@ -32,11 +30,6 @@ class RequestClient(BaseClient):
 
     REQUEST_BASEURL = 'requests'
 
-    def __init__(self, rucio_host=None, auth_host=None, account=None, ca_cert=None,
-                 auth_type=None, creds=None, timeout=600, user_agent='rucio-clients', vo=None):
-        super(RequestClient, self).__init__(rucio_host, auth_host, account, ca_cert,
-                                            auth_type, creds, timeout, user_agent, vo=vo)
-
     def list_request_by_did(self, name, rse, scope=None):
         """Return latest request details for a DID
 
@@ -53,7 +46,7 @@ class RequestClient(BaseClient):
 
         path = '/'.join(['requests', quote_plus(scope), quote_plus(name), rse])
         url = build_url(choice(self.list_hosts), path=path)
-        r = self._send_request(url, type='GET')
+        r = self._send_request(url, type_='GET')
 
         if r.status_code == codes.ok:
             return next(self._load_json_data(r))
