@@ -12,14 +12,13 @@
  - Vincent Garonne <vgaronne@gmail.com>, 2017-2018
  - Brian Bockelman <bbockelm@cse.unl.edu>, 2017
  - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
+ - Thomas Beermann <thomas.beermann@cern.ch>, 2021
 
  PY3K COMPATIBLE
 """
 
 import json
 import os
-import logging
-import sys
 
 from functools import wraps
 
@@ -38,14 +37,6 @@ from rucio.common.exception import UndefinedPolicy
 REGION = make_region().configure('dogpile.cache.memory',
                                  expiration_time=1800)
 
-logging.basicConfig(stream=sys.stdout,
-                    level=getattr(logging,
-                                  config_get('common',
-                                             'loglevel',
-                                             raise_exception=False,
-                                             default='DEBUG').upper()),
-                    format='%(asctime)s\t%(process)d\t%(levelname)s\t%(message)s')
-
 
 def get_policy():
     policy = REGION.get('policy')
@@ -74,10 +65,11 @@ def get_lifetime_policy():
     lifetime_dict = REGION.get('lifetime_dict')
     if isinstance(lifetime_dict, NoValue):
         lifetime_dict = {'data': [], 'mc': [], 'valid': [], 'other': []}
+        lifetime_dir = '/opt/rucio/etc/policies'
         try:
             lifetime_dir = config_get('lifetime', 'directory')
         except (NoSectionError, NoOptionError):
-            lifetime_dir = '/opt/rucio/etc/policies'
+            pass
         for dtype in ['data', 'mc', 'valid', 'other']:
             input_file_name = '%s/config_%s.json' % (lifetime_dir, dtype)
             if os.path.isfile(input_file_name):
