@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2021 CERN
+# Copyright 2012-2022 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,30 +23,22 @@
 # - Brandon White <bjwhite@fnal.gov>, 2019
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
+# - Gabriele Fronzé <sucre.91@hotmail.it>, 2021
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2022
 
 from dogpile.cache import make_region
 
+from rucio.common.cache import make_region_memcached
+from rucio.common.utils import is_client
 from rucio.rse import rsemanager
-from rucio.common import config
-from os import environ
 
-if 'RUCIO_CLIENT_MODE' not in environ:
-    if config.config_has_section('database'):
-        setattr(rsemanager, 'CLIENT_MODE', False)
-        setattr(rsemanager, 'SERVER_MODE', True)
-    elif config.config_has_section('client'):
-        setattr(rsemanager, 'CLIENT_MODE', True)
-        setattr(rsemanager, 'SERVER_MODE', False)
-    else:
-        setattr(rsemanager, 'CLIENT_MODE', False)
-        setattr(rsemanager, 'SERVER_MODE', True)
+if is_client():
+    setattr(rsemanager, 'CLIENT_MODE', True)
+    setattr(rsemanager, 'SERVER_MODE', False)
 else:
-    if environ['RUCIO_CLIENT_MODE']:
-        setattr(rsemanager, 'CLIENT_MODE', True)
-        setattr(rsemanager, 'SERVER_MODE', False)
-    else:
-        setattr(rsemanager, 'CLIENT_MODE', False)
-        setattr(rsemanager, 'SERVER_MODE', True)
+    setattr(rsemanager, 'CLIENT_MODE', False)
+    setattr(rsemanager, 'SERVER_MODE', True)
 
 
 def get_rse_client(rse, vo='def', **kwarg):
@@ -115,8 +107,5 @@ if rsemanager.SERVER_MODE:   # pylint:disable=no-member
 
     setattr(rsemanager, '__request_rse_info', tmp_rse_info)
     setattr(rsemanager, '__get_signed_url', get_signed_url_server)
-    RSE_REGION = make_region(function_key_generator=rse_key_generator).configure(
-        'dogpile.cache.memcached',
-        expiration_time=3600,
-        arguments={'url': config.config_get('cache', 'url', False, '127.0.0.1:11211'), 'distributed_lock': True})
+    RSE_REGION = make_region_memcached(expiration_time=3600)
     setattr(rsemanager, 'RSE_REGION', RSE_REGION)
